@@ -19,6 +19,8 @@
 
 """Administration commands for maintaining Roundup trackers.
 """
+from __future__ import print_function
+
 __docformat__ = 'restructuredtext'
 
 import csv, getopt, getpass, os, re, shutil, sys, UserDict, operator
@@ -26,10 +28,16 @@ import csv, getopt, getpass, os, re, shutil, sys, UserDict, operator
 from roundup import date, hyperdb, roundupdb, init, password, token
 from roundup import __version__ as roundup_version
 import roundup.instance
-from roundup.configuration import CoreConfig
+from roundup.configuration import CoreConfig, NoConfigError
 from roundup.i18n import _
 from roundup.exceptions import UsageError
 
+# Polyglot code
+try:
+    my_input = raw_input
+except NameError:
+    my_input = input
+
 class CommandDict(UserDict.UserDict):
     """Simple dictionary that lets us do lookups using partial keys.
 
@@ -72,6 +80,7 @@ class AdminTool:
         self.tracker_home = ''
         self.db = None
         self.db_uncommitted = False
+        self.force = None
 
     def get_class(self, classname):
         """Get the class - raise an exception if it doesn't exist.
@@ -107,7 +116,7 @@ class AdminTool:
         """
         if message:
             message = _('Problem: %(message)s\n\n')%locals()
-        print _("""%(message)sUsage: roundup-admin [options] [<command> <arguments>]
+        sys.stdout.write( _("""%(message)sUsage: roundup-admin [options] [<command> <arguments>]
 
 Options:
  -i instance home  -- specify the issue tracker "home directory" to administer
@@ -128,48 +137,47 @@ Help:
  roundup-admin help                       -- this help
  roundup-admin help <command>             -- command-specific help
  roundup-admin help all                   -- all available help
-""")%locals()
+""")%locals())
         self.help_commands()
 
     def help_commands(self):
         """List the commands available with their help summary.
         """
-        print _('Commands:'),
+        sys.stdout.write( _('Commands: '))
         commands = ['']
-        for command in self.commands.itervalues():
+        for command in self.commands.values():
             h = _(command.__doc__).split('\n')[0]
             commands.append(' '+h[7:])
         commands.sort()
         commands.append(_(
 """Commands may be abbreviated as long as the abbreviation
 matches only one command, e.g. l == li == lis == list."""))
-        print '\n'.join(commands)
-        print
+        sys.stdout.write('\n'.join(commands) + '\n\n')
 
     def help_commands_html(self, indent_re=re.compile(r'^(\s+)\S+')):
         """ Produce an HTML command list.
         """
-        commands = sorted(self.commands.itervalues(),
+        commands = sorted(iter(self.commands.values()),
             operator.attrgetter('__name__'))
         for command in commands:
             h = _(command.__doc__).split('\n')
             name = command.__name__[3:]
             usage = h[0]
-            print """
+            print("""
 <tr><td valign=top><strong>%(name)s</strong></td>
     <td><tt>%(usage)s</tt><p>
-<pre>""" % locals()
+<pre>""" % locals())
             indent = indent_re.match(h[3])
             if indent: indent = len(indent.group(1))
             for line in h[3:]:
                 if indent:
-                    print line[indent:]
+                    print(line[indent:])
                 else:
-                    print line
-            print '</pre></td></tr>\n'
+                    print(line)
+            print('</pre></td></tr>\n')
 
     def help_all(self):
-        print _("""
+        print(_("""
 All commands (except help) require a tracker specifier. This is just
 the path to the roundup tracker you're working with. A roundup tracker
 is where roundup keeps the database and configuration file that defines
@@ -230,10 +238,10 @@ Date format examples:
   "." means "right now"
 
 Command help:
-""")
-        for name, command in self.commands.items():
-            print _('%s:')%name
-            print '   ', _(command.__doc__)
+"""))
+        for name, command in list(self.commands.items()):
+            print(_('%s:')%name)
+            print('   ', _(command.__doc__))
 
     def do_help(self, args, nl_re=re.compile('[\r\n]'),
             indent_re=re.compile(r'^(\s+)\S+')):
@@ -260,20 +268,20 @@ Command help:
         try:
             l = self.commands.get(topic)
         except KeyError:
-            print _('Sorry, no help for "%(topic)s"')%locals()
+            print(_('Sorry, no help for "%(topic)s"')%locals())
             return 1
 
         # display the help for each match, removing the docsring indent
         for name, help in l:
             lines = nl_re.split(_(help.__doc__))
-            print lines[0]
+            print(lines[0])
             indent = indent_re.match(lines[1])
             if indent: indent = len(indent.group(1))
             for line in lines[1:]:
                 if indent:
-                    print line[indent:]
+                    print(line[indent:])
                 else:
-                    print line
+                    print(line)
         return 0
 
     def listTemplates(self):
@@ -305,7 +313,7 @@ Command help:
         #
         # we're interested in where the directory containing "share" is
         templates = {}
-        for N in 2, 4, 5:
+        for N in 2, 4, 5, 6:
             path = __file__
             # move up N elements in the path
             for i in range(N):
@@ -337,10 +345,10 @@ Command help:
 
     def help_initopts(self):
         templates = self.listTemplates()
-        print _('Templates:'), ', '.join(templates)
+        print(_('Templates:'), ', '.join(templates))
         import roundup.backends
         backends = roundup.backends.list_backends()
-        print _('Back ends:'), ', '.join(backends)
+        print(_('Back ends:'), ', '.join(backends))
 
     def do_install(self, tracker_home, args):
         ''"""Usage: install [template [backend [key=val[,key=val]]]]
@@ -379,36 +387,35 @@ Command help:
         # check for both old- and new-style configs
         if list(filter(os.path.exists, [config_ini_file,
                 os.path.join(tracker_home, 'config.py')])):
-            ok = raw_input(_(
+            if not self.force:
+                ok = my_input(_(
 """WARNING: There appears to be a tracker in "%(tracker_home)s"!
 If you re-install it, you will lose all the data!
 Erase it? Y/N: """) % locals())
-            if ok.strip().lower() != 'y':
-                return 0
+                if ok.strip().lower() != 'y':
+                    return 0
 
             # clear it out so the install isn't confused
             shutil.rmtree(tracker_home)
 
         # select template
         templates = self.listTemplates()
-        template = len(args) > 1 and args[1] or ''
-        if template not in templates:
-            print _('Templates:'), ', '.join(templates)
-        while template not in templates:
-            template = raw_input(_('Select template [classic]: ')).strip()
-            if not template:
-                template = 'classic'
+        template = self._get_choice(
+            list_name=_('Templates:'),
+            prompt=_('Select template'),
+            options=templates,
+            argument=len(args) > 1 and args[1] or '',
+            default='classic')
 
         # select hyperdb backend
         import roundup.backends
         backends = roundup.backends.list_backends()
-        backend = len(args) > 2 and args[2] or ''
-        if backend not in backends:
-            print _('Back ends:'), ', '.join(backends)
-        while backend not in backends:
-            backend = raw_input(_('Select backend [anydbm]: ')).strip()
-            if not backend:
-                backend = 'anydbm'
+        backend = self._get_choice(
+            list_name=_('Back ends:'),
+            prompt=_('Select backend'),
+            options=backends,
+            argument=len(args) > 2 and args[2] or '',
+            default='anydbm')
         # XXX perform a unit test based on the user's selections
 
         # Process configuration file definitions
@@ -416,31 +423,31 @@ Erase it? Y/N: """) % locals())
             try:
                 defns = dict([item.split("=") for item in args[3].split(",")])
             except:
-                print _('Error in configuration settings: "%s"') % args[3]
+                print(_('Error in configuration settings: "%s"') % args[3])
                 raise
         else:
             defns = {}
 
+        defns['rdbms_backend'] = backend
         # install!
         init.install(tracker_home, templates[template]['path'], settings=defns)
-        init.write_select_db(tracker_home, backend)
 
-        print _("""
+        print(_("""
 ---------------------------------------------------------------------------
  You should now edit the tracker configuration file:
-   %(config_file)s""") % {"config_file": config_ini_file}
+   %(config_file)s""") % {"config_file": config_ini_file})
 
         # find list of options that need manual adjustments
         # XXX config._get_unset_options() is marked as private
         #   (leading underscore).  make it public or don't care?
         need_set = CoreConfig(tracker_home)._get_unset_options()
         if need_set:
-            print _(" ... at a minimum, you must set following options:")
+            print(_(" ... at a minimum, you must set following options:"))
             for section in need_set:
-                print "   [%s]: %s" % (section, ", ".join(need_set[section]))
+                print("   [%s]: %s" % (section, ", ".join(need_set[section])))
 
         # note about schema modifications
-        print _("""
+        print(_("""
  If you wish to modify the database schema,
  you should also edit the schema file:
    %(database_config_file)s
@@ -454,19 +461,46 @@ Erase it? Y/N: """) % locals())
 """) % {
     'database_config_file': os.path.join(tracker_home, 'schema.py'),
     'database_init_file': os.path.join(tracker_home, 'initial_data.py'),
-}
+})
         return 0
 
-    def do_genconfig(self, args):
+    def _get_choice(self, list_name, prompt, options, argument, default=None):
+        if default is None:
+            default = options[0]  # just pick the first one
+        if argument in options:
+            return argument
+        if self.force:
+            return default
+        sys.stdout.write('%s: %s\n' % (list_name, ', '.join(options)))
+        while argument not in options:
+            argument = my_input('%s [%s]: ' % (prompt, default))
+            if not argument:
+                return default
+        return argument
+
+    def do_genconfig(self, args, update=False):
         ''"""Usage: genconfig <filename>
         Generate a new tracker config file (ini style) with default values
         in <filename>.
         """
         if len(args) < 1:
             raise UsageError(_('Not enough arguments supplied'))
-        config = CoreConfig()
+        if update:
+            # load current config for writing
+            config = CoreConfig(self.tracker_home)
+        else:
+            # generate default config
+            config = CoreConfig()
         config.save(args[0])
 
+    def do_updateconfig(self, args):
+        ''"""Usage: updateconfig <filename>
+        Generate an updated tracker config file (ini style) in
+        <filename>. Use current settings from existing roundup
+        tracker in tracker home.
+        """
+        self.do_genconfig(args, update=True)
+
     def do_initialise(self, tracker_home, args):
         ''"""Usage: initialise [adminpw]
         Initialise a new Roundup tracker.
@@ -495,23 +529,20 @@ Erase it? Y/N: """) % locals())
 
         # is there already a database?
         if tracker.exists():
-            ok = raw_input(_(
+            if not self.force:
+                ok = my_input(_(
 """WARNING: The database is already initialised!
 If you re-initialise it, you will lose all the data!
 Erase it? Y/N: """))
-            if ok.strip().lower() != 'y':
-                return 0
-
-            backend = tracker.get_backend_name()
+                if ok.strip().lower() != 'y':
+                    return 0
 
             # nuke it
             tracker.nuke()
 
-            # re-write the backend select file
-            init.write_select_db(tracker_home, backend, tracker.config.DATABASE)
-
         # GO
-        tracker.init(password.Password(adminpw, config=tracker.config))
+        tracker.init(password.Password(adminpw, config=tracker.config),
+                     tx_Source = 'cli')
 
         return 0
 
@@ -535,7 +566,7 @@ Erase it? Y/N: """))
             # decode the node designator
             try:
                 classname, nodeid = hyperdb.splitDesignator(designator)
-            except hyperdb.DesignatorError, message:
+            except hyperdb.DesignatorError as message:
                 raise UsageError(message)
 
             # get the class
@@ -583,9 +614,9 @@ Erase it? Y/N: """))
                         propclassname = self.db.getclass(property.classname).classname
                         id = cl.get(nodeid, propname)
                         for i in id:
-                            print propclassname + i
+                            print(propclassname + i)
                     else:
-                        print cl.get(nodeid, propname)
+                        print(cl.get(nodeid, propname))
             except IndexError:
                 raise UsageError(_('no such %(classname)s node '
                     '"%(nodeid)s"')%locals())
@@ -593,7 +624,7 @@ Erase it? Y/N: """))
                 raise UsageError(_('no such %(classname)s property '
                     '"%(propname)s"')%locals())
         if self.separator:
-            print self.separator.join(l)
+            print(self.separator.join(l))
 
         return 0
 
@@ -613,6 +644,8 @@ Erase it? Y/N: """))
         is un-set. If the property is a multilink, you specify the linked
         ids for the multilink as comma-separated numbers (ie "1,2,3").
         """
+        import copy # needed for copying props list
+
         if len(args) < 2:
             raise UsageError(_('Not enough arguments supplied'))
         from roundup import hyperdb
@@ -629,28 +662,34 @@ Erase it? Y/N: """))
         else:
             try:
                 designators = [hyperdb.splitDesignator(x) for x in designators]
-            except hyperdb.DesignatorError, message:
+            except hyperdb.DesignatorError as message:
                 raise UsageError(message)
 
         # get the props from the args
-        props = self.props_from_args(args[1:])
+        propset = self.props_from_args(args[1:]) # parse the cli once
 
         # now do the set for all the nodes
         for classname, itemid in designators:
+            props = copy.copy(propset) # make a new copy for every designator
             cl = self.get_class(classname)
 
-            properties = cl.getprops()
-            for key, value in props.items():
+            for key, value in list(props.items()):
                 try:
+                    # You must reinitialize the props every time though.
+                    # if props['nosy'] = '+admin' initally, it gets
+                    # set to 'demo,admin' (assuming it was set to demo
+                    # in the db) after rawToHyperdb returns.
+                    # This  new value is used for all the rest of the
+                    # designators if not reinitalized.
                     props[key] = hyperdb.rawToHyperdb(self.db, cl, itemid,
                         key, value)
-                except hyperdb.HyperdbValueError, message:
+                except hyperdb.HyperdbValueError as message:
                     raise UsageError(message)
 
             # try the set
             try:
                 cl.set(itemid, **props)
-            except (TypeError, IndexError, ValueError), message:
+            except (TypeError, IndexError, ValueError) as message:
                 import traceback; traceback.print_exc()
                 raise UsageError(message)
         self.db_uncommitted = True
@@ -674,7 +713,7 @@ Erase it? Y/N: """))
         props = self.props_from_args(args[1:])
 
         # convert the user-input value to a value used for find()
-        for propname, value in props.iteritems():
+        for propname, value in props.items():
             if ',' in value:
                 values = value.split(',')
             else:
@@ -697,22 +736,22 @@ Erase it? Y/N: """))
                     id = cl.find(**props)
                     for i in id:
                         designator.append(classname + i)
-                    print self.separator.join(designator)
+                    print(self.separator.join(designator))
                 else:
-                    print self.separator.join(cl.find(**props))
+                    print(self.separator.join(cl.find(**props)))
 
             else:
                 if self.print_designator:
                     id = cl.find(**props)
                     for i in id:
                         designator.append(classname + i)
-                    print designator
+                    print(designator)
                 else:
-                    print cl.find(**props)
+                    print(cl.find(**props))
         except KeyError:
             raise UsageError(_('%(classname)s has no property '
                 '"%(propname)s"')%locals())
-        except (ValueError, TypeError), message:
+        except (ValueError, TypeError) as message:
             raise UsageError(message)
         return 0
 
@@ -733,9 +772,9 @@ Erase it? Y/N: """))
         for key in cl.properties:
             value = cl.properties[key]
             if keyprop == key:
-                print _('%(key)s: %(value)s (key property)')%locals()
+                sys.stdout.write( _('%(key)s: %(value)s (key property)\n')%locals() )
             else:
-                print _('%(key)s: %(value)s')%locals()
+                sys.stdout.write( _('%(key)s: %(value)s\n')%locals() )
 
     def do_display(self, args):
         ''"""Usage: display designator[,designator]*
@@ -754,7 +793,7 @@ Erase it? Y/N: """))
         for designator in args[0].split(','):
             try:
                 classname, nodeid = hyperdb.splitDesignator(designator)
-            except hyperdb.DesignatorError, message:
+            except hyperdb.DesignatorError as message:
                 raise UsageError(message)
 
             # get the class
@@ -764,7 +803,7 @@ Erase it? Y/N: """))
             keys = sorted(cl.properties)
             for key in keys:
                 value = cl.get(nodeid, key)
-                print _('%(key)s: %(value)s')%locals()
+                print(_('%(key)s: %(value)s')%locals())
 
     def do_create(self, args):
         ''"""Usage: create classname property=value ...
@@ -799,11 +838,11 @@ Erase it? Y/N: """))
                             'propname': key.capitalize()})
                         again = getpass.getpass(_('   %(propname)s (Again): ')%{
                             'propname': key.capitalize()})
-                        if value != again: print _('Sorry, try again...')
+                        if value != again: print(_('Sorry, try again...'))
                     if value:
                         props[key] = value
                 else:
-                    value = raw_input(_('%(propname)s (%(proptype)s): ')%{
+                    value = my_input(_('%(propname)s (%(proptype)s): ')%{
                         'propname': key.capitalize(), 'proptype': name})
                     if value:
                         props[key] = value
@@ -815,7 +854,7 @@ Erase it? Y/N: """))
             try:
                 props[propname] = hyperdb.rawToHyperdb(self.db, cl, None,
                     propname, props[propname])
-            except hyperdb.HyperdbValueError, message:
+            except hyperdb.HyperdbValueError as message:
                 raise UsageError(message)
 
         # check for the key property
@@ -826,8 +865,8 @@ Erase it? Y/N: """))
 
         # do the actual create
         try:
-            print cl.create(**props)
-        except (TypeError, IndexError, ValueError), message:
+            sys.stdout.write(cl.create(**props) + '\n')
+        except (TypeError, IndexError, ValueError) as message:
             raise UsageError(message)
         self.db_uncommitted = True
         return 0
@@ -870,11 +909,11 @@ Erase it? Y/N: """))
                     except KeyError:
                         raise UsageError(_('%(classname)s has no property '
                             '"%(propname)s"')%locals())
-                print self.separator.join(proplist)
+                print(self.separator.join(proplist))
             else:
                 # create a list of index id's since user didn't specify
                 # otherwise
-                print self.separator.join(cl.list())
+                print(self.separator.join(cl.list()))
         else:
             for nodeid in cl.list():
                 try:
@@ -882,7 +921,7 @@ Erase it? Y/N: """))
                 except KeyError:
                     raise UsageError(_('%(classname)s has no property '
                         '"%(propname)s"')%locals())
-                print _('%(nodeid)4s: %(value)s')%locals()
+                print(_('%(nodeid)4s: %(value)s')%locals())
         return 0
 
     def do_table(self, args):
@@ -959,7 +998,7 @@ Erase it? Y/N: """))
                props.append((spec, maxlen))
 
         # now display the heading
-        print ' '.join([name.capitalize().ljust(width) for name,width in props])
+        print(' '.join([name.capitalize().ljust(width) for name,width in props]))
 
         # and the table data
         for nodeid in cl.list():
@@ -977,27 +1016,38 @@ Erase it? Y/N: """))
                     value = str(nodeid)
                 f = '%%-%ds'%width
                 l.append(f%value[:width])
-            print ' '.join(l)
+            print(' '.join(l))
         return 0
 
     def do_history(self, args):
-        ''"""Usage: history designator
+        ''"""Usage: history designator [skipquiet]
         Show the history entries of a designator.
 
         A designator is a classname and a nodeid concatenated,
         eg. bug1, user10, ...
 
-        Lists the journal entries for the node identified by the designator.
+        Lists the journal entries viewable by the user for the
+        node identified by the designator. If skipquiet is the
+        second argument, journal entries for quiet properties
+        are not shown.
         """
+
         if len(args) < 1:
             raise UsageError(_('Not enough arguments supplied'))
         try:
             classname, nodeid = hyperdb.splitDesignator(args[0])
-        except hyperdb.DesignatorError, message:
+        except hyperdb.DesignatorError as message:
             raise UsageError(message)
 
+        skipquiet = False
+        if len(args) == 2:
+            if args[1] != 'skipquiet':
+                raise UsageError("Second argument is not skipquiet")
+            skipquiet = True
+
         try:
-            print self.db.getclass(classname).history(nodeid)
+            print(self.db.getclass(classname).history(nodeid,
+                                                      skipquiet=skipquiet))
         except KeyError:
             raise UsageError(_('no such class "%(classname)s"')%locals())
         except IndexError:
@@ -1049,7 +1099,7 @@ Erase it? Y/N: """))
         for designator in designators:
             try:
                 classname, nodeid = hyperdb.splitDesignator(designator)
-            except hyperdb.DesignatorError, message:
+            except hyperdb.DesignatorError as message:
                 raise UsageError(message)
             try:
                 self.db.getclass(classname).retire(nodeid)
@@ -1076,7 +1126,7 @@ Erase it? Y/N: """))
         for designator in designators:
             try:
                 classname, nodeid = hyperdb.splitDesignator(designator)
-            except hyperdb.DesignatorError, message:
+            except hyperdb.DesignatorError as message:
                 raise UsageError(message)
             try:
                 self.db.getclass(classname).restore(nodeid)
@@ -1177,8 +1227,7 @@ Erase it? Y/N: """))
                 journals.writerow(row)
             jf.close()
         if max_len > self.db.config.CSV_FIELD_SIZE:
-            print >> sys.stderr, \
-                "Warning: config csv_field_size should be at least %s"%max_len
+            print("Warning: config csv_field_size should be at least %s"%max_len, file=sys.stderr)
         return 0
 
     def do_exporttables(self, args):
@@ -1260,7 +1309,7 @@ Erase it? Y/N: """))
                 maxid = max(maxid, int(nodeid))
 
             # (print to sys.stdout here to allow tests to squash it .. ugh)
-            print >> sys.stdout
+            print(file=sys.stdout)
 
             f.close()
 
@@ -1271,7 +1320,7 @@ Erase it? Y/N: """))
             f.close()
 
             # (print to sys.stdout here to allow tests to squash it .. ugh)
-            print >> sys.stdout, 'setting', classname, maxid+1
+            print('setting', classname, maxid+1, file=sys.stdout)
 
             # set the id counter
             self.db.setid(classname, str(maxid+1))
@@ -1351,34 +1400,34 @@ Erase it? Y/N: """))
             try:
                 roles = [(args[0], self.db.security.role[args[0]])]
             except KeyError:
-                print _('No such Role "%(role)s"')%locals()
+                sys.stdout.write( _('No such Role "%(role)s"\n')%locals() )
                 return 1
         else:
             roles = list(self.db.security.role.items())
             role = self.db.config.NEW_WEB_USER_ROLES
             if ',' in role:
-                print _('New Web users get the Roles "%(role)s"')%locals()
+                sys.stdout.write( _('New Web users get the Roles "%(role)s"\n')%locals() )
             else:
-                print _('New Web users get the Role "%(role)s"')%locals()
+                sys.stdout.write( _('New Web users get the Role "%(role)s"\n')%locals() )
             role = self.db.config.NEW_EMAIL_USER_ROLES
             if ',' in role:
-                print _('New Email users get the Roles "%(role)s"')%locals()
+                sys.stdout.write( _('New Email users get the Roles "%(role)s"\n')%locals() )
             else:
-                print _('New Email users get the Role "%(role)s"')%locals()
+                sys.stdout.write( _('New Email users get the Role "%(role)s"\n')%locals() )
         roles.sort()
         for rolename, role in roles:
-            print _('Role "%(name)s":')%role.__dict__
+            sys.stdout.write( _('Role "%(name)s":\n')%role.__dict__ )
             for permission in role.permissions:
                 d = permission.__dict__
                 if permission.klass:
                     if permission.properties:
-                        print _(' %(description)s (%(name)s for "%(klass)s"'
-                          ': %(properties)s only)')%d
+                        sys.stdout.write( _(' %(description)s (%(name)s for "%(klass)s"' +
+                          ': %(properties)s only)\n')%d )
                     else:
-                        print _(' %(description)s (%(name)s for "%(klass)s" '
-                            'only)')%d
+                        sys.stdout.write( _(' %(description)s (%(name)s for "%(klass)s" ' +
+                            'only)\n')%d )
                 else:
-                    print _(' %(description)s (%(name)s)')%d
+                    sys.stdout.write( _(' %(description)s (%(name)s)\n')%d )
         return 0
 
 
@@ -1402,10 +1451,10 @@ Erase it? Y/N: """))
         the habit.
         """
         if getattr(self.db, 'db_version_updated'):
-            print _('Tracker updated')
+            print(_('Tracker updated'))
             self.db_uncommitted = True
         else:
-            print _('No migration action required')
+            print(_('No migration action required'))
         return 0
 
     def run_command(self, args):
@@ -1425,64 +1474,70 @@ Erase it? Y/N: """))
             self.help_commands()
             self.help_all()
             return 0
-        if command == 'config':
-            self.do_config(args[1:])
-            return 0
 
         # figure what the command is
         try:
             functions = self.commands.get(command)
         except KeyError:
             # not a valid command
-            print _('Unknown command "%(command)s" ("help commands" for a '
-                'list)')%locals()
+            print(_('Unknown command "%(command)s" ("help commands" for a '
+                'list)')%locals())
             return 1
 
         # check for multiple matches
         if len(functions) > 1:
-            print _('Multiple commands match "%(command)s": %(list)s')%{'command':
-                command, 'list': ', '.join([i[0] for i in functions])}
+            print(_('Multiple commands match "%(command)s": %(list)s')%{'command':
+                command, 'list': ', '.join([i[0] for i in functions])})
             return 1
         command, function = functions[0]
 
         # make sure we have a tracker_home
         while not self.tracker_home:
-            self.tracker_home = raw_input(_('Enter tracker home: ')).strip()
+            if not self.force:
+                self.tracker_home = my_input(_('Enter tracker home: ')).strip()
+            else:
+                self.tracker_home = os.curdir
 
         # before we open the db, we may be doing an install or init
         if command == 'initialise':
             try:
                 return self.do_initialise(self.tracker_home, args)
-            except UsageError, message:
-                print _('Error: %(message)s')%locals()
+            except UsageError as message:
+                print(_('Error: %(message)s')%locals())
                 return 1
         elif command == 'install':
             try:
                 return self.do_install(self.tracker_home, args)
-            except UsageError, message:
-                print _('Error: %(message)s')%locals()
+            except UsageError as message:
+                print(_('Error: %(message)s')%locals())
                 return 1
 
         # get the tracker
         try:
             tracker = roundup.instance.open(self.tracker_home)
-        except ValueError, message:
+        except ValueError as message:
             self.tracker_home = ''
-            print _("Error: Couldn't open tracker: %(message)s")%locals()
+            print(_("Error: Couldn't open tracker: %(message)s")%locals())
+            return 1
+        except NoConfigError as message:
+            self.tracker_home = ''
+            print(_("Error: Couldn't open tracker: %(message)s")%locals())
             return 1
 
         # only open the database once!
         if not self.db:
             self.db = tracker.open('admin')
 
+        self.db.tx_Source = 'cli'
+
         # do the command
         ret = 0
         try:
             ret = function(args[1:])
-        except UsageError, message:
-            print _('Error: %(message)s')%locals()
-            print
-            print function.__doc__
+        except UsageError as message:
+            print(_('Error: %(message)s')%locals())
+            print()
+            print(function.__doc__)
             ret = 1
         except:
             import traceback
@@ -1493,28 +1548,31 @@ Erase it? Y/N: """))
     def interactive(self):
         """Run in an interactive mode
         """
-        print _('Roundup %s ready for input.\nType "help" for help.'
-            % roundup_version)
+        print(_('Roundup %s ready for input.\nType "help" for help.'
+            % roundup_version))
         try:
             import readline
         except ImportError:
-            print _('Note: command history and editing not available')
+            print(_('Note: command history and editing not available'))
 
         while 1:
             try:
-                command = raw_input(_('roundup> '))
+                command = my_input(_('roundup> '))
             except EOFError:
-                print _('exit...')
+                print(_('exit...'))
                 break
             if not command: continue
-            args = token.token_split(command)
+            try:
+                args = token.token_split(command)
+            except ValueError:
+                continue	# Ignore invalid quoted token
             if not args: continue
             if args[0] in ('quit', 'exit'): break
             self.run_command(args)
 
         # exit.. check for transactions
         if self.db and self.db_uncommitted:
-            commit = raw_input(_('There are unsaved changes. Commit them (y/N)? '))
+            commit = my_input(_('There are unsaved changes. Commit them (y/N)? '))
             if commit and commit[0].lower() == 'y':
                 self.db.commit()
         return 0
@@ -1522,7 +1580,7 @@ Erase it? Y/N: """))
     def main(self):
         try:
             opts, args = getopt.getopt(sys.argv[1:], 'i:u:hcdsS:vV')
-        except getopt.GetoptError, e:
+        except getopt.GetoptError as e:
             self.usage(str(e))
             return 1
 
@@ -1543,7 +1601,7 @@ Erase it? Y/N: """))
                 self.usage()
                 return 0
             elif opt == '-v':
-                print '%s (python %s)'%(roundup_version, sys.version.split()[0])
+                print('%s (python %s)'%(roundup_version, sys.version.split()[0]))
                 return 0
             elif opt == '-V':
                 self.verbose = 1
