@@ -38,7 +38,7 @@ date_re = re.compile(r'''^
     |(?P<a>\d\d?)[/-](?P<b>\d\d?))?              # or mm-dd
     (?P<n>\.)?                                   # .
     (((?P<H>\d?\d):(?P<M>\d\d))?(:(?P<S>\d\d?(\.\d+)?))?)?  # hh:mm:ss
-    (?P<o>[\d\smywd\-+]+)?                       # offset
+    (?:(?P<tz>\s?[+-]\d{4})|(?P<o>[\d\smywd\-+]+))? # time-zone offset, offset
 $''', re.VERBOSE)
 serialised_date_re = re.compile(r'''
     (\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d?(\.\d+)?)
@@ -145,6 +145,7 @@ def get_timezone(tz):
 
 def _utc_to_local(y,m,d,H,M,S,tz):
     TZ = get_timezone(tz)
+    S = min(S, 59.999)
     frac = S - int(S)
     dt = datetime.datetime(y, m, d, H, M, int(S), tzinfo=UTC)
     y,m,d,H,M,S = dt.astimezone(TZ).timetuple()[:6]
@@ -157,6 +158,19 @@ def _local_to_utc(y,m,d,H,M,S,tz):
     y,m,d,H,M,S = TZ.localize(dt).utctimetuple()[:6]
     return (y,m,d,H,M,S)
 
+def test_ini(t):
+    """ Monkey-patch to make doctest think it's always time t:
+    """
+    u = Date.now 
+    d = datetime.datetime.strptime (t, '%Y-%m-%d.%H:%M:%S.%f')
+    Date.now = lambda x : d
+    return u
+
+def test_fin(u):
+    """ Undo monkey patch above
+    """
+    Date.now = u
+
 class Date:
     '''
     As strings, date-and-time stamps are specified with the date in
@@ -165,7 +179,7 @@ class Date:
     and are fairly readable when printed. An example of a valid stamp is
     "2000-06-24.13:03:59". We'll call this the "full date format". When
     Timestamp objects are printed as strings, they appear in the full date
-    format with the time always given in GMT. The full date format is
+    format with the time always given in UTC. The full date format is
     always exactly 19 characters long.
 
     For user input, some partial forms are also permitted: the whole time
@@ -175,18 +189,46 @@ class Date:
     care of these conversions. In the following examples, suppose that yyyy
     is the current year, mm is the current month, and dd is the current day
     of the month; and suppose that the user is on Eastern Standard Time.
+
+    Note that Date conversion from user inputs will use the local
+    timezone, either from the database user (some database schemas have
+    a timezone property for a user) or from a default in the roundup
+    configuration. Roundup will store all times in UTC in the database
+    but display the time to the user in their local timezone as
+    configured. In the following examples the timezone correction for
+    Eastern Standard Time (GMT-5, no DST) will be applied explicitly via
+    an offset, but times are given in UTC in the output.
+
     Examples::
 
-      "2000-04-17" means <Date 2000-04-17.00:00:00>
-      "01-25" means <Date yyyy-01-25.00:00:00>
-      "2000-04-17.03:45" means <Date 2000-04-17.08:45:00>
-      "08-13.22:13" means <Date yyyy-08-14.03:13:00>
-      "11-07.09:32:43" means <Date yyyy-11-07.14:32:43>
-      "14:25" means <Date yyyy-mm-dd.19:25:00>
-      "8:47:11" means <Date yyyy-mm-dd.13:47:11>
-      "2003" means <Date 2003-01-01.00:00:00>
-      "2003-06" means <Date 2003-06-01.00:00:00>
-      "." means "right now"
+
+        make doctest think it's always 2000-06-26.00:34:02:
+        >>> u = test_ini('2000-06-26.00:34:02.0')
+
+        >>> Date("2000-04-17-0500")
+        <Date 2000-04-17.05:00:00.000>
+        >>> Date("01-25-0500")
+        <Date 2000-01-25.05:00:00.000>
+        >>> Date("2000-04-17.03:45-0500")
+        <Date 2000-04-17.08:45:00.000>
+        >>> Date("08-13.22:13-0500")
+        <Date 2000-08-14.03:13:00.000>
+        >>> Date("11-07.09:32:43-0500")
+        <Date 2000-11-07.14:32:43.000>
+        >>> Date("14:25-0500")
+        <Date 2000-06-26.19:25:00.000>
+        >>> Date("8:47:11-0500")
+        <Date 2000-06-26.13:47:11.000>
+        >>> Date("2003 -0500")
+        <Date 2003-01-01.05:00:00.000>
+        >>> Date("2003-06 -0500")
+        <Date 2003-06-01.05:00:00.000>
+
+        "." means "right now":
+        >>> Date(".")
+        <Date 2000-06-26.00:34:02.000>
+
+        >>> test_fin(u)
 
     The Date class should understand simple date expressions of the form
     stamp + interval and stamp - interval. When adding or subtracting
@@ -196,40 +238,68 @@ class Date:
     2000-08-04 (rather than trying to decide whether 1m 10d means 38 or 40
     or 41 days).  Example usage::
 
+        make doctest think it's always 2000-06-26.00:34:02:
+        >>> u = test_ini('2000-06-26.00:34:02.0')
+
         >>> Date(".")
-        <Date 2000-06-26.00:34:02>
+        <Date 2000-06-26.00:34:02.000>
         >>> _.local(-5)
-        "2000-06-25.19:34:02"
+        <Date 2000-06-25.19:34:02.000>
         >>> Date(". + 2d")
-        <Date 2000-06-28.00:34:02>
+        <Date 2000-06-28.00:34:02.000>
         >>> Date("1997-04-17", -5)
-        <Date 1997-04-17.00:00:00>
+        <Date 1997-04-17.05:00:00.000>
         >>> Date("01-25", -5)
-        <Date 2000-01-25.00:00:00>
+        <Date 2000-01-25.05:00:00.000>
         >>> Date("08-13.22:13", -5)
-        <Date 2000-08-14.03:13:00>
+        <Date 2000-08-14.03:13:00.000>
         >>> Date("14:25", -5)
-        <Date 2000-06-25.19:25:00>
+        <Date 2000-06-26.19:25:00.000>
 
     The date format 'yyyymmddHHMMSS' (year, month, day, hour,
     minute, second) is the serialisation format returned by the serialise()
     method, and is accepted as an argument on instatiation.
 
-    The date class handles basic arithmetic::
+    In addition, a timezone specifier can be appended to the date format.
+    The timezone specifier is a sign ("+" or "-") followed by a 4-digit
+    number as in the RFC 2822 date format.
+    The first two digits indicate the number of hours, while the last two
+    digits indicate the number of minutes the time is offset from
+    Coordinated Universal Time (UTC). The "+" or "-" sign indicate whether
+    the time is ahead of (east of) or behind (west of) UTC. Note that a
+    given timezone specifier *overrides* an offset given to the Date
+    constructor.  Examples::
 
+        >>> Date ("2000-08-14+0200")
+        <Date 2000-08-13.22:00:00.000>
+        >>> Date ("08-15.22:00+0200")
+        <Date 2000-08-15.20:00:00.000>
+        >>> Date ("08-15.22:47+0200")
+        <Date 2000-08-15.20:47:00.000>
+        >>> Date ("08-15.22:47+0200", offset = 5)
+        <Date 2000-08-15.20:47:00.000>
+        >>> Date ("08-15.22:47", offset = 5)
+        <Date 2000-08-15.17:47:00.000>
+
+    The date class handles basic arithmetic, but note that arithmetic
+    cannot be combined with timezone offsets (see last example)::
+
+        >>> x=test_ini('2004-04-06.22:04:20.766830')
         >>> d1=Date('.')
         >>> d1
-        <Date 2004-04-06.22:04:20.766830>
+        <Date 2004-04-06.22:04:20.767>
         >>> d2=Date('2003-07-01')
         >>> d2
-        <Date 2003-07-01.00:00:0.000000>
+        <Date 2003-07-01.00:00:00.000>
         >>> d1-d2
         <Interval + 280d 22:04:20>
         >>> i1=_
         >>> d2+i1
-        <Date 2004-04-06.22:04:20.000000>
+        <Date 2004-04-06.22:04:20.000>
         >>> d1-i1
-        <Date 2003-07-01.00:00:0.000000>
+        <Date 2003-07-01.00:00:00.000>
+
+        >>> test_fin(u)
     '''
 
     def __init__(self, spec='.', offset=0, add_granularity=False,
@@ -269,15 +339,22 @@ class Date:
             spec = spec.get_tuple()
         try:
             y,m,d,H,M,S,x,x,x = spec
+            S = min(S, 59.999)
             frac = S - int(S)
             self.year, self.month, self.day, self.hour, self.minute, \
                 self.second = _local_to_utc(y, m, d, H, M, S, offset)
             # we lost the fractional part
             self.second = self.second + frac
-            if str(self.second) == '60.0': self.second = 59.9
+            # making sure we match the precision of serialise()
+            self.second = min(self.second, 59.999)
         except:
             raise ValueError, 'Unknown spec %r' % (spec,)
 
+    def now(self):
+        """ To be able to override for testing
+        """
+        return datetime.datetime.utcnow()
+
     def set(self, spec, offset=0, date_re=date_re,
             serialised_re=serialised_date_re, add_granularity=False):
         ''' set the date to the value in spec
@@ -295,9 +372,9 @@ class Date:
         # not serialised data, try usual format
         m = date_re.match(spec)
         if m is None:
-            raise ValueError, self._('Not a date spec: '
-                '"yyyy-mm-dd", "mm-dd", "HH:MM", "HH:MM:SS" or '
-                '"yyyy-mm-dd.HH:MM:SS.SSS"')
+            raise ValueError, self._('Not a date spec: %r '
+                '("yyyy-mm-dd", "mm-dd", "HH:MM", "HH:MM:SS" or '
+                '"yyyy-mm-dd.HH:MM:SS.SSS")' % spec)
 
         info = m.groupdict()
 
@@ -321,7 +398,7 @@ class Date:
                 raise ValueError(self._('Could not determine granularity'))
 
         # get the current date as our default
-        dt = datetime.datetime.utcnow()
+        dt = self.now()
         y,m,d,H,M,S,x,x,x = dt.timetuple()
         S += dt.microsecond/1000000.
 
@@ -352,6 +429,8 @@ class Date:
                 S = float(info['S'])
             adjust = True
 
+        if info.get('tz', None):
+            offset = 0
 
         # now handle the adjustment of hour
         frac = S - int(S)
@@ -372,6 +451,13 @@ class Date:
                     '"yyyy-mm-dd", "mm-dd", "HH:MM", "HH:MM:SS" or '
                     '"yyyy-mm-dd.HH:MM:SS.SSS"')%(spec,)
 
+        if info.get('tz', None):
+            tz     = info ['tz'].strip ()
+            sign   = [-1,1][tz[0] == '-']
+            minute = int (tz[3:], 10)
+            hour   = int (tz[1:3], 10)
+            self.applyInterval(Interval((0, 0, 0, hour, minute, 0), sign=sign))
+
         # adjust by added granularity
         if add_granularity:
             self.applyInterval(add_granularity)
@@ -543,6 +629,13 @@ class Date:
             self.second, 0, 0, 0)
 
     def serialise(self):
+        """ Return serialised string for self's datetime.
+
+        Uses '%06.3f' as format for self.second, which therefor
+        must be <=59.999 to work. Otherwise it will be rounded
+        to 60.000.
+
+        """
         return '%04d%02d%02d%02d%02d%06.3f'%(self.year, self.month,
             self.day, self.hour, self.minute, self.second)
 
@@ -593,10 +686,13 @@ class Interval:
       "0:04:33" means four minutes and 33 seconds
 
     Example usage:
+        make doctest think it's always 2000-06-26.00:34:02:
+        >>> u = test_ini('2000-06-26.00:34:02.0')
+
         >>> Interval("  3w  1  d  2:00")
         <Interval + 22d 2:00>
         >>> Date(". + 2d") + Interval("- 3w")
-        <Date 2000-06-07.00:34:02>
+        <Date 2000-06-07.00:34:02.000>
         >>> Interval('1:59:59') + Interval('00:00:01')
         <Interval + 2:00>
         >>> Interval('2:00') + Interval('- 00:00:01')
@@ -605,10 +701,16 @@ class Interval:
         <Interval + 6m>
         >>> Interval('1:00')/2
         <Interval + 0:30>
+
+        [number of days between 2000-06-26.00:34:02 and 2003-03-18
         >>> Interval('2003-03-18')
-        <Interval + [number of days between now and 2003-03-18]>
+        <Interval - 995d>
+
+        [number of days between 2000-06-26.00:34:02 and 2003-03-14
         >>> Interval('-4d 2003-03-18')
-        <Interval + [number of days between now and 2003-03-14]>
+        <Interval - 991d>
+
+        >>> test_fin(u)
 
     Interval arithmetic is handled in a couple of special ways, trying
     to cater for the most common cases. Fundamentally, Intervals which
@@ -681,8 +783,9 @@ class Interval:
         if not m:
             m = interval_re.match(spec)
             if not m:
-                raise ValueError, self._('Not an interval spec:'
-                    ' [+-] [#y] [#m] [#w] [#d] [[[H]H:MM]:SS] [date spec]')
+                raise ValueError, self._('Not an interval spec: "%s"'
+                    ' ([+-] [#y] [#m] [#w] [#d] [[[H]H:MM]:SS] [date spec])'
+                    % spec)
         else:
             allowdate = 0
 
@@ -703,8 +806,9 @@ class Interval:
 
         # make sure it's valid
         if not valid and not info['D']:
-            raise ValueError, self._('Not an interval spec:'
-                ' [+-] [#y] [#m] [#w] [#d] [[[H]H:MM]:SS]')
+            raise ValueError, self._('Not an interval spec: "%s"'
+                ' ([+-] [#y] [#m] [#w] [#d] [[[H]H:MM]:SS])'
+                % spec)
 
         if self.week:
             self.day = self.day + self.week*7
@@ -1001,24 +1105,57 @@ class Range:
 
     Examples (consider local time is Sat Mar  8 22:07:48 EET 2003)::
 
-        >>> Range("from 2-12 to 4-2")
+        make doctest think it's always 2000-06-26.00:34:02:
+        >>> u = test_ini('2003-03-08.20:07:48.0')
+
+        >>> Range("from 2-12 to 4-2", Date)
         <Range from 2003-02-12.00:00:00 to 2003-04-02.00:00:00>
 
-        >>> Range("18:00 TO +2m")
+        >>> Range("18:00 to +2m", Date)
         <Range from 2003-03-08.18:00:00 to 2003-05-08.20:07:48>
 
-        >>> Range("12:00")
-        <Range from 2003-03-08.12:00:00 to None>
-
-        >>> Range("tO +3d")
+        >>> Range("tO +3d", Date)
         <Range from None to 2003-03-11.20:07:48>
 
-        >>> Range("2002-11-10; 2002-12-12")
+        >>> Range("12:00 to", Date)
+        <Range from 2003-03-08.12:00:00 to None>
+
+        >>> Range("12:00;", Date)
+        <Range from 2003-03-08.12:00:00 to None>
+
+        >>> Range("2002-11-10; 2002-12-12", Date)
         <Range from 2002-11-10.00:00:00 to 2002-12-12.00:00:00>
 
-        >>> Range("; 20:00 +1d")
+        >>> Range("; 20:00 +1d", Date)
         <Range from None to 2003-03-09.20:00:00>
 
+        >>> Range("from 2003-02-16", Date)
+        <Range from 2003-02-16.00:00:00 to None>
+
+        >>> Range("2003-02-16;", Date)
+        <Range from 2003-02-16.00:00:00 to None>
+
+        Granularity tests:
+
+        >>> Range("12:00", Date)
+        <Range from 2003-03-08.12:00:00 to 2003-03-08.12:00:59>
+
+        >>> Range("2003-03-08", Date)
+        <Range from 2003-03-08.00:00:00 to 2003-03-08.23:59:59>
+
+        >>> test_fin(u)
+
+        Range of Interval tests
+
+        >>> Range ("from 0:50 to 2:00", Interval)
+        <Range from + 0:50 to + 2:00>
+        >>> Range ("from 0:50 to 1d 2:00", Interval)
+        <Range from + 0:50 to + 1d 2:00>
+        >>> Range ("from 5:50", Interval)
+        <Range from + 5:50 to None>
+        >>> Range ("to 0:05", Interval)
+        <Range from None to + 0:05>
+
     """
     def __init__(self, spec, Type, allow_granularity=True, **params):
         """Initializes Range of type <Type> from given <spec> string.
@@ -1031,8 +1168,9 @@ class Range:
         class instance.
         """
         self.range_type = Type
-        re_range = r'(?:^|from(.+?))(?:to(.+?)$|$)'
-        re_geek_range = r'(?:^|(.+?));(?:(.+?)$|$)'
+        re_range = r'^(?:from)?(.+?)?to(.+?)?$'
+        re_range_no_to = r'^from(.+)(.)?$'
+        re_geek_range = r'^(.+?)?;(.+?)?$'
         # Check which syntax to use
         if ';' in spec:
             # Geek
@@ -1040,6 +1178,8 @@ class Range:
         else:
             # Native english
             m = re.search(re_range, spec.strip(), re.IGNORECASE)
+            if not m :
+                m = re.search(re_range_no_to, spec.strip(), re.IGNORECASE)
         if m:
             self.from_value, self.to_value = m.groups()
             if self.from_value:
