@@ -31,14 +31,6 @@ Incoming messages are examined for multiple parts:
    special handling of the first text part) if unpack_rfc822 is set in
    the mailgw config section.
 
-Summary
--------
-The "summary" property on message nodes is taken from the first non-quoting
-section in the message body. The message body is divided into sections by
-blank lines. Sections where the second and all subsequent lines begin with
-a ">" or "|" character are considered "quoting sections". The first line of
-the first non-quoting section becomes the summary of the message.
-
 Addresses
 ---------
 All of the addresses in the To: and Cc: headers of the incoming message are
@@ -54,10 +46,26 @@ nodes with no passwords.
 
 Actions
 -------
-The subject line of the incoming message is examined to determine whether
-the message is an attempt to create a new item or to discuss an existing
-item. A designator enclosed in square brackets is sought as the first thing
-on the subject line (after skipping any "Fwd:" or "Re:" prefixes).
+The subject line of the incoming message is examined to determine
+whether the message is an attempt to create a new item, discuss an
+existing item, or execute some other command.
+
+If the subject consists of one of the following (case insensitive),
+the corresponding action is taken:
+
+help
+  Respond with an explanation of this interface.
+
+If the subject contains the following, the corresponding action is
+taken:
+
+-- key <OneTimeKey>
+  Complete an in-progress user registration.
+
+If the email is not a command, it is either a new item or a message
+associated with an existing item.  A designator enclosed in square
+brackets is sought as the first thing on the subject line (after
+skipping any "Fwd:" or "Re:" prefixes).
 
 If an item designator (class name and id number) is found there, the newly
 created "msg" node is added to the "messages" property for that item, and
@@ -68,6 +76,14 @@ of that class with its "messages" property initialized to contain the new
 "msg" node and its "files" property initialized to contain any new "file"
 nodes.
 
+Summary
+-------
+The "summary" property on message nodes is taken from the first non-quoting
+section in the message body. The message body is divided into sections by
+blank lines. Sections where the second and all subsequent lines begin with
+a ">" or "|" character are considered "quoting sections". The first line of
+the first non-quoting section becomes the summary of the message.
+
 Triggers
 --------
 Both cases may trigger detectors (in the first case we are calling the
@@ -80,11 +96,12 @@ __docformat__ = 'restructuredtext'
 
 import string, re, os, mimetools, cStringIO, smtplib, socket, binascii, quopri
 import time, random, sys, logging
-import traceback, rfc822
+import traceback
+import email.utils
 
 from anypy.email_ import decode_header
 
-from roundup import configuration, hyperdb, date, password, rfc2822, exceptions
+from roundup import configuration, hyperdb, date, password, exceptions
 from roundup.mailer import Mailer, MessageSendError
 from roundup.i18n import _
 from roundup.hyperdb import iter_roles
@@ -102,15 +119,15 @@ class MailGWError(ValueError):
 class MailUsageError(ValueError):
     pass
 
-class MailUsageHelp(Exception):
+class MailUsageHelp(BaseException):
     """ We need to send the help message to the user. """
     pass
 
-class Unauthorized(Exception):
+class Unauthorized(BaseException):
     """ Access denied """
     pass
 
-class IgnoreMessage(Exception):
+class IgnoreMessage(BaseException):
     """ A general class of message that we should ignore. """
     pass
 class IgnoreBulk(IgnoreMessage):
@@ -147,7 +164,7 @@ def getparam(str, param):
         if '=' in f:
             i = f.index('=')
             if f[:i].strip().lower() == param:
-                return rfc822.unquote(f[i+1:].strip())
+                return email.utils.unquote(f[i+1:].strip())
     return None
 
 def gpgh_key_getall(key, attr):
@@ -175,17 +192,17 @@ def check_pgp_sigs(sigs, gpgctx, author, may_be_unsigned=False):
                 # try to narrow down the actual problem to give a more useful
                 # message in our bounce
                 if sig.summary & pyme.constants.sigsum.KEY_MISSING:
-                    raise MailUsageError, \
-                        _("Message signed with unknown key: %s") % sig.fpr
+                    raise MailUsageError( \
+                        _("Message signed with unknown key: %s") % sig.fpr)
                 elif sig.summary & pyme.constants.sigsum.KEY_EXPIRED:
-                    raise MailUsageError, \
-                        _("Message signed with an expired key: %s") % sig.fpr
+                    raise MailUsageError( \
+                        _("Message signed with an expired key: %s") % sig.fpr)
                 elif sig.summary & pyme.constants.sigsum.KEY_REVOKED:
-                    raise MailUsageError, \
-                        _("Message signed with a revoked key: %s") % sig.fpr
+                    raise MailUsageError( \
+                        _("Message signed with a revoked key: %s") % sig.fpr)
                 else:
-                    raise MailUsageError, \
-                        _("Invalid PGP signature detected.")
+                    raise MailUsageError( \
+                        _("Invalid PGP signature detected."))
 
     # we couldn't find a key belonging to the author of the email
     if sigs:
@@ -319,8 +336,9 @@ class Message(mimetools.Message):
             data = self.fp.read()
 
         # Encode message to unicode
-        charset = rfc2822.unaliasCharset(self.getparam("charset"))
+        charset = self.getparam("charset")
         if charset:
+            charset = charset.lower().replace("windows-", 'cp')
             # Do conversion only if charset specified - handle
             # badly-specified charsets
             edata = unicode(data, charset, 'replace').encode('utf-8')
@@ -364,7 +382,7 @@ class Message(mimetools.Message):
     #   Only if configured in [mailgw] unpack_rfc822
 
     def extract_content(self, parent_type=None, ignore_alternatives=False,
-        unpack_rfc822=False):
+            unpack_rfc822=False, html2text=None):
         """Extract the body and the attachments recursively.
 
            If the content is hidden inside a multipart/alternative part,
@@ -374,24 +392,43 @@ class Message(mimetools.Message):
         content_type = self.gettype()
         content = None
         attachments = []
+        html_part = False
 
         if content_type == 'text/plain':
             content = self.getbody()
+        elif content_type == 'text/html' and html2text:
+            # if user allows html conversion run this.
+            content = html2text(self.getbody())
+            attachments.append(self.as_attachment())
+            html_part = True
         elif content_type[:10] == 'multipart/':
-            content_found = bool (content)
-            ig = ignore_alternatives and not content_found
+            content_found = False
+            ig = ignore_alternatives
+            html_part_found = False
             for part in self.getparts():
-                new_content, new_attach = part.extract_content(content_type,
-                    not content and ig, unpack_rfc822)
+                new_content, new_attach, html_part = part.extract_content(
+                     content_type, not content and ig, unpack_rfc822,
+                    html2text)
 
                 # If we haven't found a text/plain part yet, take this one,
                 # otherwise make it an attachment.
                 if not content:
                     content = new_content
                     cpart   = part
+                    if html_part:
+                        html_part_found = True
                 elif new_content:
-                    if content_found or content_type != 'multipart/alternative':
+                    if html_part:
+                        # attachment should be added elsewhere.
+                        pass
+                    elif content_found or content_type != 'multipart/alternative':
                         attachments.append(part.text_as_attachment())
+                    elif html_part_found:
+                        # text/plain part found after html
+                        # text/html already stored as attachment,
+                        # so just use the text as the content.
+                        content = new_content
+                        cpart   = part
                     else:
                         # if we have found a text/plain in the current
                         # multipart/alternative and find another one, we
@@ -407,12 +444,13 @@ class Message(mimetools.Message):
                 attachments.extend(new_attach)
             if ig and content_type == 'multipart/alternative' and content:
                 attachments = []
+            html_part = False
         elif unpack_rfc822 and content_type == 'message/rfc822':
             s = cStringIO.StringIO(self.getbody())
             m = Message(s)
             ig = ignore_alternatives and not content
-            new_content, attachments = m.extract_content(m.gettype(), ig,
-                unpack_rfc822)
+            new_content, attachments, html_part = m.extract_content(m.gettype(), ig,
+                    unpack_rfc822, html2text)
             attachments.insert(0, m.text_as_attachment())
         elif (parent_type == 'multipart/signed' and
               content_type == 'application/pgp-signature'):
@@ -420,7 +458,7 @@ class Message(mimetools.Message):
             pass
         else:
             attachments.append(self.as_attachment())
-        return content, attachments
+        return content, attachments, html_part
 
     def text_as_attachment(self):
         """Return first text/plain part as Message"""
@@ -538,8 +576,12 @@ class parsedMessage:
         self.has_prefix = False
         self.matches = dict.fromkeys(['refwd', 'quote', 'classname',
                                  'nodeid', 'title', 'args', 'argswhole'])
-        self.from_list = message.getaddrlist('resent-from') \
-                         or message.getaddrlist('from')
+        self.keep_real_from = self.config['EMAIL_KEEP_REAL_FROM']
+        if self.keep_real_from:
+            self.from_list = message.getaddrlist('from')
+        else:
+            self.from_list = message.getaddrlist('resent-from') \
+                          or message.getaddrlist('from')
         self.pfxmode = self.config['MAILGW_SUBJECT_PREFIX_PARSING']
         self.sfxmode = self.config['MAILGW_SUBJECT_SUFFIX_PARSING']
         # these are filled in by subsequent parsing steps
@@ -630,33 +672,33 @@ Emails to Roundup trackers must include a Subject: line!
 
             tmpsubject = tmpsubject[m.end():]
 
-        # Match the title of the subject
-        # if we've not found a valid classname prefix then force the
-        # scanning to handle there being a leading delimiter
-        title_re = r'(?P<title>%s[^%s]*)'%(
-            not self.matches['classname'] and '.' or '', delim_open)
-        m = re.match(title_re, tmpsubject.strip(), re.IGNORECASE)
-        if m:
-            self.matches.update(m.groupdict())
-            tmpsubject = tmpsubject[len(self.matches['title']):] # Consume title
-
-        if self.matches['title']:
-            self.matches['title'] = self.matches['title'].strip()
-        else:
-            self.matches['title'] = ''
-
-        # strip off the quotes that dumb emailers put around the subject, like
-        #      Re: "[issue1] bla blah"
-        if self.matches['quote'] and self.matches['title'].endswith('"'):
-            self.matches['title'] = self.matches['title'][:-1]
-        
-        # Match any arguments specified
-        args_re = r'(?P<argswhole>%s(?P<args>.+?)%s)?'%(delim_open,
-            delim_close)
+        # Match any arguments specified *from the end*
+        # Optionally match and strip quote at the end that dumb mailers
+        # may put there, e.g.
+        #      Re: "[issue1] bla blah [<args>]"
+        q = ''
+        if self.matches['quote']:
+            q = '"?'
+        args_re = r'(?P<argswhole>%s(?P<args>[^%s]*)%s)%s$'%(delim_open,
+            delim_close, delim_close, q)
         m = re.search(args_re, tmpsubject.strip(), re.IGNORECASE|re.VERBOSE)
         if m:
             self.matches.update(m.groupdict())
+            tmpsubject = tmpsubject [:m.start()]
+        else:
+            self.matches['argswhole'] = self.matches['args'] = None
 
+        # The title of the subject is the remaining tmpsubject.
+        self.matches ['title'] = tmpsubject.strip ()
+
+        # strip off the quotes that dumb emailers put around the subject, like
+        #      Re: "[issue1] bla blah"
+        # but only if we didn't match arguments at the end (which would
+        # already have consumed the quote after the subject)
+        if self.matches['quote'] and not self.matches['argswhole'] \
+           and self.matches['title'].endswith('"'):
+            self.matches['title'] = self.matches['title'][:-1]
+        
     def rego_confirm(self):
         ''' Check for registration OTK and confirm the registration if found
         '''
@@ -1010,6 +1052,9 @@ Subject was: "%(subject)s"
                     "be PGP encrypted.")
             if self.message.pgp_signed():
                 self.message.verify_signature(author_address)
+                # signature has been verified
+                self.db.tx_Source = "email-sig-openpgp"
+
             elif self.message.pgp_encrypted():
                 # Replace message with the contents of the decrypted
                 # message for content extraction
@@ -1019,8 +1064,26 @@ Subject was: "%(subject)s"
                 encr_only = self.config.PGP_REQUIRE_INCOMING == 'encrypted'
                 encr_only = encr_only or not pgp_role()
                 self.crypt = True
-                self.message = self.message.decrypt(author_address,
-                    may_be_unsigned = encr_only)
+                try:
+                    # see if the message has a valid signature
+                    message = self.message.decrypt(author_address,
+                                                   may_be_unsigned = False)
+                    # only set if MailUsageError is not raised
+                    # indicating that we have a valid signature
+                    self.db.tx_Source = "email-sig-openpgp"
+                except MailUsageError:
+                    # if there is no signature or an error in the message
+                    # we get here. Try decrypting it again if we don't
+                    # need signatures.
+                    if encr_only:
+                        message = self.message.decrypt(author_address,
+                                               may_be_unsigned = encr_only)
+                    else:
+                        # something failed with the message decryption/sig
+                        # chain. Pass the error up.
+                        raise
+                # store the decrypted message      
+                self.message = message
             elif pgp_role():
                 raise MailUsageError, _("""
 This tracker has been configured to require all email be PGP signed or
@@ -1029,10 +1092,15 @@ encrypted.""")
     def get_content_and_attachments(self):
         ''' get the attachments and first text part from the message
         '''
+        from roundup.dehtml import dehtml
+        html2text=dehtml(self.config['MAILGW_CONVERT_HTMLTOTEXT']).html2text
+
         ig = self.config.MAILGW_IGNORE_ALTERNATIVES
-        self.content, self.attachments = self.message.extract_content(
+        self.message.instance = self.mailgw.instance
+        self.content, self.attachments, html_part = self.message.extract_content(
             ignore_alternatives=ig,
-            unpack_rfc822=self.config.MAILGW_UNPACK_RFC822)
+            unpack_rfc822=self.config.MAILGW_UNPACK_RFC822,
+            html2text=html2text )
         
 
     def create_files(self):
@@ -1095,9 +1163,8 @@ encrypted.""")
 Roundup requires the submission to be plain text. The message parser could
 not find a text/plain part to use.
 """)
-
         # parse the body of the message, stripping out bits as appropriate
-        summary, content = parseContent(self.content, config=self.config)
+        summary, content = parseContent(self.content, config=self.config, is_new_issue = not bool(self.nodeid))
         content = content.strip()
 
         if content:
@@ -1110,11 +1177,11 @@ not find a text/plain part to use.
                     recipients=self.recipients, date=date.Date('.'),
                     summary=summary, content=content,
                     messageid=messageid, inreplyto=inreplyto, **self.msg_props)
-            except exceptions.Reject, error:
-                raise MailUsageError, _("""
+            except exceptions.Reject as error:
+                raise MailUsageError(_("""
 Mail message was rejected by a detector.
 %(error)s
-""") % locals()
+""") % locals())
             # allowed to attach the message to the existing node?
             if self.nodeid and not self.db.security.hasPermission('Edit',
                     self.author, self.classname, 'messages'):
@@ -1154,7 +1221,8 @@ Mail message was rejected by a detector.
                             'property %(prop)s of class %(classname)s.'
                             ) % locals()
                 self.nodeid = self.cl.create(**self.props)
-        except (TypeError, IndexError, ValueError, exceptions.Reject), message:
+        except (TypeError, IndexError, ValueError, exceptions.Reject) as message:
+            self.mailgw.logger.exception("Rejecting email due to node creation error:")
             raise MailUsageError, _("""
 There was a problem with the message you sent:
    %(message)s
@@ -1330,7 +1398,7 @@ class MailGW:
                 server.login_cram_md5(user, password)
             else:
                 server.login(user, password)
-        except imaplib.IMAP4.error, e:
+        except imaplib.IMAP4.error as e:
             self.logger.exception('IMAP login failure')
             return 1
 
@@ -1345,7 +1413,7 @@ class MailGW:
                 return 1
             try:
                 numMessages = int(data[0])
-            except ValueError, value:
+            except ValueError as value:
                 self.logger.error('Invalid message count from mailbox %r'%
                     data[0])
                 return 1
@@ -1384,6 +1452,15 @@ class MailGW:
         '''Read a series of messages from the specified POP server.
         '''
         import getpass, poplib, socket
+        # Monkey-patch poplib to have a large line-limit
+        # Seems that in python2.7 poplib applies a line-length limit not
+        # just to the lines that take care of the pop3 protocol but also
+        # to all email content
+        # See, e.g.,
+        # https://readlist.com/lists/python.org/python-list/69/346982.html
+        # https://stackoverflow.com/questions/30976106/python-poplib-error-proto-line-too-+long?utm_medium=organic&utm_source=google_rich_qa&utm_campaign=google_rich_qa
+        if 0 < getattr (poplib, '_MAXLINE', -1) < 100*1024:
+            poplib._MAXLINE = 100*1024
         try:
             if not user:
                 user = raw_input('User: ')
@@ -1446,7 +1523,7 @@ class MailGW:
         self.parsed_message = None
         crypt = False
         sendto = message.getaddrlist('resent-from')
-        if not sendto:
+        if not sendto or self.instance.config['EMAIL_KEEP_REAL_FROM']:
             sendto = message.getaddrlist('from')
         if not sendto:
             # very bad-looking message - we don't even know who sent it
@@ -1476,14 +1553,16 @@ class MailGW:
             return self.handle_message(message)
         except MailUsageHelp:
             # bounce the message back to the sender with the usage message
+            self.logger.debug("MailUsageHelp raised, bouncing.")
             fulldoc = '\n'.join(string.split(__doc__, '\n')[2:])
             m = ['']
             m.append('\n\nMail Gateway Help\n=================')
             m.append(fulldoc)
             self.mailer.bounce_message(message, [sendto[0][1]], m,
                 subject="Mail Gateway Help")
-        except MailUsageError, value:
+        except MailUsageError as value:
             # bounce the message back to the sender with the usage message
+            self.logger.debug("MailUsageError raised, bouncing.")
             fulldoc = '\n'.join(string.split(__doc__, '\n')[2:])
             m = ['']
             m.append(str(value))
@@ -1493,8 +1572,9 @@ class MailGW:
                 message = self.parsed_message.message
                 crypt = self.parsed_message.crypt
             self.mailer.bounce_message(message, [sendto[0][1]], m, crypt=crypt)
-        except Unauthorized, value:
+        except Unauthorized as value:
             # just inform the user that he is not authorized
+            self.logger.debug("Unauthorized raised, bouncing.")
             m = ['']
             m.append(str(value))
             if self.parsed_message:
@@ -1537,6 +1617,9 @@ class MailGW:
         '''
         # get database handle for handling one email
         self.db = self.instance.open ('admin')
+
+        self.db.tx_Source = "email"
+
         try:
             return self._handle_message(message)
         finally:
@@ -1639,7 +1722,7 @@ def setPropArrayFromString(self, cl, propString, nodeid=None):
         # extract the property name and value
         try:
             propname, value = prop.split('=')
-        except ValueError, message:
+        except ValueError as message:
             errors.append(_('not of form [arg=value,value,...;'
                 'arg=value,value,...]'))
             return (errors, props)
@@ -1648,7 +1731,7 @@ def setPropArrayFromString(self, cl, propString, nodeid=None):
         try:
             props[propname] = hyperdb.rawToHyperdb(self.db, cl, nodeid,
                 propname, value)
-        except hyperdb.HyperdbValueError, message:
+        except hyperdb.HyperdbValueError as message:
             errors.append(str(message))
     return errors, props
 
@@ -1735,7 +1818,7 @@ def uidFromAddress(db, address, create=1, **user_props):
     else:
         return 0
 
-def parseContent(content, keep_citations=None, keep_body=None, config=None):
+def parseContent(content, keep_citations=None, keep_body=None, config=None, is_new_issue=False):
     """Parse mail message; return message summary and stripped content
 
     The message body is divided into sections by blank lines.
@@ -1759,8 +1842,30 @@ def parseContent(content, keep_citations=None, keep_body=None, config=None):
         config = configuration.CoreConfig()
     if keep_citations is None:
         keep_citations = config["MAILGW_KEEP_QUOTED_TEXT"]
+        if keep_citations == "new":
+            # don't strip citations if we are a new issue
+            if is_new_issue:
+                keep_citations = True
+            else:
+                keep_citations = False
+        elif keep_citations == "yes":
+            keep_citations = True
+        else:
+            keep_citations = False
+
     if keep_body is None:
         keep_body = config["MAILGW_LEAVE_BODY_UNCHANGED"]
+        if keep_body == "new":
+            # don't strip citations if we are a new issue
+            if is_new_issue:
+                keep_body = True
+            else:
+                keep_body = False
+        elif keep_body == "yes":
+            keep_body = True
+        else:
+            keep_body = False
+
     eol = config["MAILGW_EOL_RE"]
     signature = config["MAILGW_SIGN_RE"]
     original_msg = config["MAILGW_ORIGMSG_RE"]
