@@ -3,14 +3,28 @@
 # Copyright 2004 Richard Jones (richard@mechanicalcat.net)
 #
 
+DEFAULT_HOME = './demo'
+DEFAULT_TEMPLATE = 'classic'
+
+
 import sys
 
+
+# --- patch sys.path to make sure 'import roundup' finds correct version
+import os.path as osp
+
+thisdir = osp.dirname(osp.abspath(__file__))
+rootdir = osp.dirname(osp.dirname(thisdir))
+if (osp.exists(thisdir + '/__init__.py') and
+        osp.exists(rootdir + '/roundup/__init__.py')):
+    # the script is located inside roundup source code
+    sys.path.insert(0, rootdir)
+# --/
+
+
 from roundup import admin, configuration, demo, instance
 from roundup.i18n import _
 
-DEFAULT_HOME = './demo'
-DEFAULT_TEMPLATE = 'classic'
-
 def run():
     home = DEFAULT_HOME
     template = DEFAULT_TEMPLATE
