@@ -7,6 +7,19 @@
 import os
 import sys
 
+
+# --- patch sys.path to make sure 'import roundup' finds correct version
+import os.path as osp
+
+thisdir = osp.dirname(osp.abspath(__file__))
+rootdir = osp.dirname(osp.dirname(thisdir))
+if (osp.exists(thisdir + '/__init__.py') and
+        osp.exists(rootdir + '/roundup/__init__.py')):
+    # the script is located inside roundup source code
+    sys.path.insert(0, rootdir)
+# --/
+
+
 from roundup.i18n import _
 from roundup.cgi.TAL import talgettext
 
