@@ -19,6 +19,22 @@
 """
 __docformat__ = 'restructuredtext'
 
+
+# --- patch sys.path to make sure 'import roundup' finds correct version
+import sys
+import os.path as osp
+
+import logging
+
+thisdir = osp.dirname(osp.abspath(__file__))
+rootdir = osp.dirname(osp.dirname(thisdir))
+if (osp.exists(thisdir + '/__init__.py') and
+        osp.exists(rootdir + '/roundup/__init__.py')):
+    # the script is located inside roundup source code
+    sys.path.insert(0, rootdir)
+# --/
+
+
 import errno, cgi, getopt, os, socket, sys, traceback, urllib, time
 import ConfigParser, BaseHTTPServer, SocketServer, StringIO
 
@@ -190,7 +206,7 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             self.inner_run_cgi()
         except client.NotFound:
             self.send_error(404, self.path)
-        except client.Unauthorised, message:
+        except client.Unauthorised as message:
             self.send_error(403, '%s (%s)'%(self.path, message))
         except:
             exc, val, tb = sys.exc_info()
@@ -218,7 +234,7 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
                     self.wfile.write(cgitb.breaker())
                     ts = time.ctime()
                     self.wfile.write('''<p>%s: An error occurred. Please check
-                    the server log for more infomation.</p>'''%ts)
+                    the server log for more information.</p>'''%ts)
                     # out to the logfile
                     print 'EXCEPTION AT', ts
                     traceback.print_exc()
@@ -263,12 +279,12 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
     def inner_run_cgi(self):
         ''' This is the inner part of the CGI handling
         '''
-        rest = self.path
 
-        # file-like object for the favicon.ico file information
-        favicon_fileobj = None
+        # self.path is /some/path?with&all=stuff
+        if self.path == '/favicon.ico':
+            # file-like object for the favicon.ico file information
+            favicon_fileobj = None
 
-        if rest == '/favicon.ico':
             # check to see if a custom favicon was specified, and set
             # favicon_fileobj to the input file
             if self.CONFIG is not None:
@@ -300,10 +316,12 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
 
             return
 
-        i = rest.rfind('?')
+        i = self.path.rfind('?')
         if i >= 0:
-            rest, query = rest[:i], rest[i+1:]
+            # rest starts with /, query is without ?
+            rest, query = self.path[:i], self.path[i+1:]
         else:
+            rest = self.path
             query = ''
 
         # no tracker - spit out the index
@@ -320,7 +338,9 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             self.send_response(301)
             # redirect - XXX https??
             protocol = 'http'
-            url = '%s://%s%s/'%(protocol, self.headers['host'], self.path)
+            url = '%s://%s%s/'%(protocol, self.headers['host'], rest)
+            if query:
+               url += '?' + query
             self.send_header('Location', url)
             self.end_headers()
             self.wfile.write('Moved Permanently')
@@ -357,9 +377,50 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
             env['HTTP_HOST'] = self.headers ['host']
         except KeyError:
             env['HTTP_HOST'] = ''
+        # https://tools.ietf.org/html/draft-ietf-appsawg-http-forwarded-10
+        # headers.
+        xfh = self.headers.getheader('X-Forwarded-Host', None)
+        if xfh:
+            # If behind a proxy, this is the hostname supplied
+            # via the Host header to the proxy. Used by core code.
+            # Controlled by the CSRF settings.
+            env['HTTP_X-FORWARDED-HOST'] = xfh
+        xff = self.headers.getheader('X-Forwarded-For', None)
+        if xff:
+            # xff is a list of ip addresses for original client/proxies:
+            # X-Forwarded-For: clientIP, proxy1IP, proxy2IP
+            # May not be trustworthy. Do not use in core without
+            # config option to control its use.
+            # Made available for extensions if the user trusts it.
+            # E.g. you may wish to disable recaptcha validation extension
+            # if the ip of the client matches 172.16.0.0.
+            env['HTTP_X-FORWARDED-FOR'] = xff
+        xfp = self.headers.getheader('X-Forwarded-Proto', None)
+        if xfp:
+            # xfp is the protocol (http/https) seen by proxies in the
+            # path of the request. I am not sure if there is only
+            # one value or multiple, but I suspect multiple
+            # is possible so:
+            # X-Forwarded-Proto: https, http
+            # is expected if the path is:
+            #    client -> proxy1 -> proxy2 -> back end server
+            # an proxy1 is an SSL terminator.
+            # May not be trustworthy. Do not use in core without
+            # config option to control its use.
+            # Made available for extensions if the user trusts it.
+            env['HTTP_X-FORWARDED-PROTO'] = xfp
         if os.environ.has_key('CGI_SHOW_TIMING'):
             env['CGI_SHOW_TIMING'] = os.environ['CGI_SHOW_TIMING']
         env['HTTP_ACCEPT_LANGUAGE'] = self.headers.get('accept-language')
+        referer = self.headers.get('Referer')
+        if referer:
+            env['HTTP_REFERER'] = referer
+        origin = self.headers.get('Origin')
+        if origin:
+            env['HTTP_ORIGIN'] = origin
+        xrw = self.headers.get('x-requested-with')
+        if xrw:
+            env['HTTP_X-REQUESTED-WITH'] = xrw
         range = self.headers.getheader('range')
         if range:
             env['HTTP_RANGE'] = range
@@ -378,12 +439,20 @@ class RoundupRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):
     def log_message(self, format, *args):
         ''' Try to *safely* log to stderr.
         '''
-        try:
-            BaseHTTPServer.BaseHTTPRequestHandler.log_message(self,
-                format, *args)
-        except IOError:
-            # stderr is no longer viable
-            pass
+        if self.CONFIG['LOGHTTPVIALOGGER']:
+            logger = logging.getLogger('roundup.http')
+
+            logger.info("%s - - [%s] %s" %
+                        (self.client_address[0],
+                         self.log_date_time_string(),
+                         format%args))
+        else:
+            try:
+                BaseHTTPServer.BaseHTTPRequestHandler.log_message(self,
+                                                         format, *args)
+            except IOError:
+                # stderr is no longer viable
+                pass
 
     def start_response(self, headers, response):
         self.send_response(response)
@@ -493,6 +562,11 @@ class ServerConfig(configuration.Config):
             (configuration.BooleanOption, "log_hostnames", "no",
                 "Log client machine names instead of IP addresses "
                 "(much slower)"),
+            (configuration.BooleanOption, "loghttpvialogger", "no",
+                "Have http(s) request logging done via python logger module.\n"
+                "If set to yes the python logging module is used with "
+                "qualname\n'roundup.http'. Otherwise logging is done to "
+                "stderr or the file\nspecified using the -l/logfile option."),
             (configuration.NullableFilePathOption, "pidfile", "",
                 "File to which the server records "
                 "the process id of the daemon.\n"
@@ -516,7 +590,7 @@ class ServerConfig(configuration.Config):
             "Option name identifies the tracker and will appear in the URL.\n"
             "Option value is tracker home directory path.\n"
             "The path may be either absolute or relative\n"
-            "to the directory containig this config file."),
+            "to the directory containing this config file."),
     )
 
     # options recognized by config
@@ -531,6 +605,7 @@ class ServerConfig(configuration.Config):
         "log_hostnames": "N",
         "multiprocess": "t:",
         "template": "i:",
+        "loghttpvialogger": 'L',
         "ssl": "s",
         "pem": "e:",
     }
@@ -647,7 +722,7 @@ class ServerConfig(configuration.Config):
             if self["SSL"]:
                 kwargs['ssl_pem'] = self["PEM"]
             httpd = server_class(*args, **kwargs)
-        except socket.error, e:
+        except socket.error as e:
             if e[0] == errno.EADDRINUSE:
                 raise socket.error, \
                     _("Unable to bind to port %s, port already in use.") \
@@ -668,7 +743,7 @@ else:
     # allow the win32
     import win32service
 
-    class SvcShutdown(Exception):
+    class SvcShutdown(BaseException):
         pass
 
     class RoundupService(win32serviceutil.ServiceFramework):
@@ -748,6 +823,7 @@ Options:
  -N            log client machine names instead of IP addresses (much slower)
  -i <fname>    set tracker index template
  -s            enable SSL
+ -L            http request logging uses python logging (roundup.http)
  -e <fname>    PEM file containing SSL key and certificate
  -t <mode>     multiprocess mode (default: %(mp_def)s).
                Allowed values: %(mp_types)s.
@@ -850,7 +926,7 @@ def run(port=undefined, success_message=None):
     try:
         (optlist, args) = config.getopt(sys.argv[1:],
             short_options, ("help", "version", "save-config",))
-    except (getopt.GetoptError, configuration.ConfigurationError), e:
+    except (getopt.GetoptError, configuration.ConfigurationError) as e:
         usage(str(e))
         return
 
