@@ -4,7 +4,7 @@ places in Roundup code.
 
 __docformat__ = 'restructuredtext'
 
-import os, time, sys
+import os, time, sys, re
 
 class TruthDict:
     '''Returns True for valid keys, False for others.
@@ -31,11 +31,11 @@ class PrioList:
     full list interface.
     Implementation: We manage a "sorted" status and sort on demand.
     Appending to the list will require re-sorting before use.
-    >>> p = PrioList ()
-    >>> for i in 5,7,1,-1 :
-    ...  p.append (i)
+    >>> p = PrioList()
+    >>> for i in 5,7,1,-1:
+    ...  p.append(i)
     ...
-    >>> for k in p :
+    >>> for k in p:
     ...  print k
     ...
     -1
@@ -49,14 +49,14 @@ class PrioList:
         self.sorted = True
 
     def append(self, item):
-        self.list.append (item)
+        self.list.append(item)
         self.sorted = False
 
     def __iter__(self):
-        if not self.sorted :
-            self.list.sort ()
+        if not self.sorted:
+            self.list.sort()
             self.sorted = True
-        return iter (self.list)
+        return iter(self.list)
 
 class Progress:
     '''Progress display for console applications.
