@@ -2,7 +2,9 @@
 '''
 
 import shutil
+import os
 
+from roundup import date
 from roundup import hyperdb
 from roundup import roundupdb
 from roundup import security
@@ -12,7 +14,6 @@ from roundup.backends import back_anydbm
 from roundup.backends import indexer_dbm
 from roundup.backends import sessions_dbm
 from roundup.backends import indexer_common
-from roundup.hyperdb import *
 from roundup.support import ensureParentsExist
 
 def new_config(debug=False):
@@ -32,7 +33,10 @@ def create(journaltag, create=True, debug=False):
     # load standard schema
     schema = os.path.join(os.path.dirname(__file__),
         '../share/roundup/templates/classic/schema.py')
-    vars = dict(globals())
+    vars = hyperdb.__dict__
+    vars['Class'] = Class
+    vars['FileClass'] = FileClass
+    vars['IssueClass'] = IssueClass
     vars['db'] = db
     execfile(schema, vars)
     initial_data = os.path.join(os.path.dirname(__file__),
@@ -42,7 +46,8 @@ def create(journaltag, create=True, debug=False):
     execfile(initial_data, vars)
 
     # load standard detectors
-    dirname = os.path.join(os.path.dirname(__file__),
+    thisdir = os.path.dirname(__file__)
+    dirname = os.path.join(thisdir,
         '../share/roundup/templates/classic/detectors')
     for fn in os.listdir(dirname):
         if not fn.endswith('.py'): continue
@@ -50,6 +55,10 @@ def create(journaltag, create=True, debug=False):
         execfile(os.path.join(dirname, fn), vars)
         vars['init'](db)
 
+    vars = {}
+    execfile(os.path.join(thisdir, "tx_Source_detector.py"), vars)
+    vars['init'](db)
+
     '''
     status = Class(db, "status", name=String())
     status.setkey("name")
@@ -173,6 +182,9 @@ class Database(back_anydbm.Database):
       modified. Do some sort of conflict checking on the dirty stuff.
     - perhaps detect write collisions (related to above)?
     """
+
+    dbtype = "memorydb"
+
     def __init__(self, config, journaltag=None):
         self.config, self.journaltag = config, journaltag
         self.classes = {}
@@ -194,6 +206,7 @@ class Database(back_anydbm.Database):
         self.newnodes = {}      # keep track of the new nodes by class
         self.destroyednodes = {}# keep track of the destroyed nodes by class
         self.transactions = []
+        self.tx_Source = None
 
     def filename(self, classname, nodeid, property=None, create=0):
         shutil.copyfile(__file__, __file__+'.dummy')
