@@ -1,4 +1,4 @@
-#-*- encoding: utf8 -*-
+#-*- encoding: utf-8 -*-
 import unittest
 
 from roundup import mailer
@@ -12,13 +12,4 @@ class EncodingTestCase(unittest.TestCase):
             '=?iso8859-1?q?caf=E9?= <ascii@test.com>')
         a('as"ii', 'ascii@test.com', 'iso8859-1', '"as\\"ii" <ascii@test.com>')
 
-def test_suite():
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(EncodingTestCase))
-    return suite
-
-if __name__ == '__main__':
-    runner = unittest.TextTestRunner()
-    unittest.main(testRunner=runner)
-
 # vim: set et sts=4 sw=4 :
