@@ -1,17 +1,15 @@
 import os, unittest, shutil
+
 from db_test_base import setupTracker
+from .test_dates import skip_pytz
+
 
 class UserAuditorTest(unittest.TestCase):
     def setUp(self):
         self.dirname = '_test_user_auditor'
         self.instance = setupTracker(self.dirname)
         self.db = self.instance.open('admin')
-
-        try:
-            import pytz
-            self.pytz = True
-        except ImportError:
-            self.pytz = False
+        self.db.tx_Source = "cli"
 
         self.db.user.create(username='kyle', address='kyle@example.com',
             realname='Kyle Broflovski', roles='User')
@@ -20,7 +18,7 @@ class UserAuditorTest(unittest.TestCase):
         self.db.close()
         try:
             shutil.rmtree(self.dirname)
-        except OSError, error:
+        except OSError as error:
             if error.errno not in (errno.ENOENT, errno.ESRCH): raise
 
     def testBadTimezones(self):
@@ -33,19 +31,21 @@ class UserAuditorTest(unittest.TestCase):
         self.assertRaises(ValueError, self.db.user.set, userid, timezone='-24')
         self.assertRaises(ValueError, self.db.user.set, userid, timezone='-3000')
 
-        if self.pytz:
-            try:
-                from pytz import UnknownTimeZoneError
-            except:
-                UnknownTimeZoneError = ValueError
-            self.assertRaises(UnknownTimeZoneError, self.db.user.set, userid, timezone='MiddleOf/Nowhere')
+    @skip_pytz
+    def testBadTimezonesPyTZ(self):
+        userid = self.db.user.lookup('kyle')
+
+        try:
+            from pytz import UnknownTimeZoneError
+        except:
+            UnknownTimeZoneError = ValueError
+
+        self.assertRaises(UnknownTimeZoneError, self.db.user.set, userid,
+                          timezone='MiddleOf/Nowhere')
 
     def testGoodTimezones(self):
         self.db.user.create(username='test_user01', timezone='12')
 
-        if self.pytz:
-            self.db.user.create(username='test_user02', timezone='MST')
-
         userid = self.db.user.lookup('kyle')
 
         # TODO: roundup should accept non-integer offsets since those are valid
@@ -56,8 +56,12 @@ class UserAuditorTest(unittest.TestCase):
         self.db.user.set(userid, timezone='23')
         self.db.user.set(userid, timezone='0')
 
-        if self.pytz:
-            self.db.user.set(userid, timezone='US/Eastern')
+    @skip_pytz
+    def testGoodTimezonesPyTZ(self):
+        userid = self.db.user.lookup('kyle')
+
+        self.db.user.create(username='test_user02', timezone='MST')
+        self.db.user.set(userid, timezone='US/Eastern')
 
     def testBadEmailAddresses(self):
         userid = self.db.user.lookup('kyle')
@@ -98,13 +102,4 @@ class UserAuditorTest(unittest.TestCase):
         # check for all-whitespace (treat as no role)
         self.db.user.set(userid, roles='   ')
 
-def test_suite():
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(UserAuditorTest))
-    return suite
-
-if __name__ == '__main__':
-    runner = unittest.TextTestRunner()
-    unittest.main(testRunner=runner)
-
 # vim: filetype=python sts=4 sw=4 et si
